#!/usr/bin/perl

use DataOneLdap;
use strict;

use Getopt::Long();

sub usage {
   my $message = $_[0];
   if (defined $message && length $message) {
      $message .= "\n"
         unless $message =~ /\n$/;
   }

   my $command = $0;
   $command =~ s#^.*/##;

   print STDERR (
      $message,
      "usage: $command [--help] [--id nodeId]\n" 
   );

   die("\n")
}

my $subject;
my $help;
my $mesg;

Getopt::Long::GetOptions(
   'subject=s' => \$subject,
   'help!' => \$help
) or usage("Invalid commmand line options.");
 
usage() if (defined $help);

usage("A subject must be defined") unless (defined $subject);

my $d1Ldap = new DataOneLdap();

my $ldap = $d1Ldap->connect();

$mesg = $ldap->search( filter=>"(&(cn=$subject)(objectClass=d1Principal))", 
                       base=>"dc=cilogon,dc=org");

my @entries = $mesg->entries;
foreach my $entry (@entries) 
	{
	my $dn = $entry->dn();
	print "DN = $dn\n";
	$ldap->modify($dn, replace => {'isVerified', 'TRUE'});
	}

$d1Ldap->disconnect();

