#!/usr/bin/perl

use strict;
use Getopt::Long();
use File::Basename;
use File::Spec::Unix;
use File::stat;

## If files have not been archived to a safe location, then do so. Otherwise,
## there will not be anything to compare modifications to

#	TXT FILES TO COMPARE VIA DIFF
my @compareFiles=(
	"/etc/init.d/tomcat7",
	"/etc/default/tomcat7",
	"/var/lib/tomcat7/conf/server.xml",
	"/var/lib/tomcat7/conf/catalina.properties",
	"/etc/ldap/slapd.conf",
	"/etc/ldap/ldap.conf",
	"/etc/dataone/ldapService.properties",
	"/etc/dataone/node.properties",
	"/usr/lib/jvm/java-1.7.0-openjdk-amd64/jre/lib/security/java.security",
	"/etc/apache2/ports.conf",
	"/etc/apache2/mods-available/jk.conf",
	"/etc/apache2/sites-available/cn-ssl",
	"/etc/libapache2-mod-jk/workers.properties",
	"/etc/postgresql/9.l/main/pg_hba.conf",
	"/etc/postgresql/9.1/main/postgresql.conf",
	"/etc/dataone/portal/hazelcast.xml",
	"/var/lib/tomcat7/webapps/portal/WEB-INF/web.xml",
	"/var/lib/tomcat7/webapps/portal/WEB-INF/client.xml",
	"/etc/dataone/process/dataoneHazelcast.properties",
	"/etc/dataone/storage/hazelcast.xml",
	"/var/lib/tomcat6/webapps/metacat/WEB-INF/metacat.properties");

	
#	DIRECTORY STRUCTURE TO COMPARE
my @compareDirectories=("/etc/ldap/slapd.d");
	
my $archiveDirectory="/opt/diff-archive";

sub diffFiles
	{
	die("compareBranches requires 2 arguments, only " . scalar(@_) . " provided") if (scalar(@_) != 2);
	my ($origFile, $newFile) = @_;
	my @result = `diff -Bbq $origFile $newFile`;
	if ($? > 0)
		{
		my @full_results = `diff $origFile $newFile`;
		print "@full_results\n";
		return 0;
		}
#	return 0 if ($? > 0);
	
    my $origFileSB = stat($origFile);
    my $newFileSB = stat($newFile);
    unless ( ($origFileSB->mode == $newFileSB->mode) && ($origFileSB->uid == $newFileSB->uid) && ($origFileSB->gid == $newFileSB->gid) )
	{
	print "$origFile\t". $origFileSB->mode . " " . $origFileSB->uid . " " . $origFileSB->gid . "\n";
	print "$newFile\t". $newFileSB->mode . " " . $newFileSB->uid . " " . $newFileSB->gid . "\n";
	return 0;
	}
    return 1;
	}
	
sub archiveFile
{
	my ($archiveFile) = @_;
	my $fullPath  = dirname ("$archiveFile");	
	my $rtn = `mkdir --parents $archiveDirectory/$fullPath`;
	return 0 if ($? > 0);

	$rtn = `cp -a $archiveFile $archiveDirectory/$archiveFile`;
	return 0 if ($? > 0);
	return 1;
}

sub archiveDir
{
	my ($archiveDir) = @_;
	my $rtn = `mkdir --parents $archiveDirectory/$archiveDir`;
	return 0 if ($? > 0);
		
	$rtn = `cp -a $archiveDir/* $archiveDirectory/$archiveDir`;
	return 0 if ($? > 0);
	return 1;

}

##### i like defining the main subroutine, especially if there is the possibility of
##### other subroutines being a part of this namespace
main:	{
	if (-e $archiveDirectory  &&  -d $archiveDirectory)
		{
		# compare the files
		for my $archFile (@compareFiles)
			{
				print "something went wrong with $archFile\n" unless (diffFiles($archFile, "$archiveDirectory/$archFile"));
			}
		# compare the directories
		##### read in all the files in the branch, that we are concerned about
		##### excluded are files name .project, .classpath (eclipse files), .class and nbactions.xml
		##### and the directories .settings (netbeans) and .svn
		#####  tmp/.lock 
		for my $archDir (@compareDirectories) 
			{
			my @archDirFiles;
			open (ARCH_DATA, "find $archDir -type f  -print |")   or die "Couldn't execute program: $!";
			while ( defined( my $line = <ARCH_DATA> )  ) 
				{
				chomp($line);
				push (@archDirFiles, $line) unless $line =~ /~$/;
				}
			close ARCH_DATA;
			
			##### make certain all the branch files are in the trunk
			foreach my $archFile (@archDirFiles)
				{
				unless (-e "$archiveDirectory/$archFile")
					{
					print("MISSING FILE in $archiveDirectory/$archFile\n");
					}
				}
		
			foreach my $archFile (@archDirFiles)
				{
				print "something went wrong with $archFile\n" unless (diffFiles($archFile, "$archiveDirectory/$archFile"));
				}
			}
		print "REMEMBER TO REMOVE THE DIRECTORY $archiveDirectory AFTER ALL COMPARISONS ARE COMPLETE!!!!\n";
		}
	else
		{
		for my $archDir (@compareDirectories)
			{
				print "something went wrong with $archDir\n" unless (archiveDir($archDir));
			}
		# archive the files
		for my $archFile (@compareFiles)
			{
				print "something went wrong with $archFile\n" unless (archiveFile($archFile));
			}
		}
}
# make certain the archive directory exists and is writable


